package ru.dbotthepony.mc.otm.capability

import net.minecraftforge.energy.IEnergyStorage
import net.minecraftforge.items.IItemHandler
import ru.dbotthepony.mc.otm.core.math.Decimal

/**
 * Attempts to safely exchange/move item between slots of two handlers
 *
 * @return pair of new (advanced) [sourceSlot] and [destinationSlot]
 */
fun moveBetweenSlots(source: IItemHandler, sourceSlot: Int, destination: IItemHandler, destinationSlot: Int): Pair<Int, Int> {
	val getItem = source.extractItem(sourceSlot, Int.MAX_VALUE, true)

	if (getItem.isEmpty) {
		return sourceSlot + 1 to destinationSlot
	} else {
		val leftover = destination.insertItem(destinationSlot, getItem, true)

		if (leftover.count == getItem.count) {
			return sourceSlot to destinationSlot + 1
		} else {
			val getItem2 = source.extractItem(sourceSlot, getItem.count - leftover.count, true)

			if (getItem2.isEmpty) {
				return sourceSlot + 1 to destinationSlot
			} else {
				val leftover2 = destination.insertItem(destinationSlot, getItem2, true)

				if (leftover2.isEmpty) {
					source.extractItem(sourceSlot, getItem2.count, false)
					destination.insertItem(destinationSlot, getItem2, false)

					if (getItem2.count == getItem.count) {
						return sourceSlot + 1 to destinationSlot
					} else {
						return sourceSlot to destinationSlot
					}
				}
			}
		}
	}

	return sourceSlot to destinationSlot
}

@Suppress("name_shadowing")
fun moveEnergy(source: IEnergyStorage, destination: IEnergyStorage, amount: Decimal = Decimal.LONG_MAX_VALUE, simulate: Boolean): Decimal {
	val extracted = source.extractEnergy(amount, true)

	if (extracted.isPositive) {
		val received = destination.receiveEnergy(extracted, true)

		if (received.isPositive) {
			val extracted = source.extractEnergy(received, true)

			if (extracted.isPositive) {
				if (simulate) {
					return extracted
				}

				val received = destination.receiveEnergy(extracted, false)
				return source.extractEnergy(received, false)
			}
		}
	}

	return Decimal.ZERO
}
